/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.submarine.server.api.job;

import org.apache.submarine.server.api.spec.JobSpec;

/**
 * The Generic Machine Learning Job in Submarine.
 */
public class Job {
  private JobId jobId;
  private String name;
  private String identifier;

  /**
   * Get the job instance
   * @param jobId job id
   * @param name job name
   * @param identifier identifier
   * @return object
   */
  public static Job newInstance(JobId jobId, String name, String identifier) {
    Job job = new Job();
    job.setJobId(jobId);
    job.setName(name);
    job.setIdentifier(identifier);
    return job;
  }

  /**
   * Get the job id which is unique in submarine
   * @return job id
   */
  public JobId getJobId() {
    return jobId;
  }

  /**
   * Set the job id which generated by submarine
   * @param jobId job id
   */
  public void setJobId(JobId jobId) {
    this.jobId = jobId;
  }

  /**
   * Get the job name which specified by user through the {@link JobSpec}
   * @return the job name
   */
  public String getName() {
    return name;
  }

  /**
   * Set the job name which specified by user
   * @param name job name
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * The unique identifier for the job, used to retire the job info from the cluster management
   * system.
   *
   * In YARN cluster it best to set the ApplicationId, and in K8s cluster it maybe the job name.
   * @return the unique identifier
   */
  public String getIdentifier() {
    return identifier;
  }

  /**
   * Set the job identifier, in YARN cluster it best to set the application id, and in K8s cluster
   * it maybe the job name.
   * @param identifier application id (YARN) or Job Name (K8s)
   */
  public void setIdentifier(String identifier) {
    this.identifier = identifier;
  }
}
